/**
 * /**
 * 
 * @file player.js
 * 
 * @brief This file contain implementation of methods required to store data
 *        from UI.
 * 
 * @author: mobile2DS
 * 
 * @date: 12.1.2014
 */

var lWarn = {
	logln : function(o) {
	}
};
var lInfo = {
	logln : function(o) {
	}
};

var LOGFILE_NAME = "sonos-controller.log";
lWarn = new Logger(LOGFILE_NAME);
lInfo = new Logger(LOGFILE_NAME);

var fShouldStore = false;

function checkAndStoreDSIDParameter(path, dsuid, dsid) {
	try {
        lInfo.logln('store dsuid at |' + path + '| value: |' + dsuid + '|' );
        var successCode = false;
        var stringValue;
        if (typeof(dsuid) == 'string') {
            Property.setProperty(path, dsuid);
            Property.setFlag(path, 'ARCHIVE', true);
            successCode = true;
        } else {
            if (typeof(dsid) == 'string') {
                stringValue = dSID2dSUID(dsid);
                if (typeof(stringValue) == "string") {
                    Property.setProperty(path, stringValue);
                    Property.setFlag(path, 'ARCHIVE', true);
                    successCode = true;
                }                
                lInfo.logln("store converted dsid");
            } else {
                lWarn.logln('Typ-Fehler beim Speichern der Property entries/' + path)
            }
        }
        return successCode;
    } catch (e) {
        lWarn.logln("Exception beim checkAndStoreDSUIDParameter " + e + " Path " + path  + " value " + dsuid);
        return false;
    }

}


/**
 * Converts the given dsid to a dsuid
 * @param {String} dSID
 * @returns {String} dSUID
 */

function dSID2dSUID(dSID) {
    try {
        //copy the first 6 bytes (12 characters to the new dsuid
        var dSUID = dSID.slice(0, 12);
        //insert 4 bytes of 0
        dSUID += '00000000';
        //append the remaining 6 bytes (12 chars) of the dsid
        dSUID += dSID.slice(12, 24);
        //finally add 1 byte of 0
        dSUID += '00';
        return dSUID;
    } catch (e) {
        lWarn.logln('Exception in store-property-helper::convertdSID2dSUID ' + e);
        return '';
    }
}



function fetchMediaDefaultConfiguration(callback) {
	 var mediaDefaultConfig = [];
	 try {
		   
		    cloudService.fetchMediaDefaultConfiguration(function (oData) {
		     
		      if (oData!=null) {
		        if (oData.ReturnCode != null && oData.ReturnCode == 0) {
		        	lInfo.logln('return for default config ' + oData.Response.MediaFileList);
		        	for (var iIndex=0;iIndex<oData.Response.MediaFileList.length;iIndex++) {
		        		var oDataObject= {
		        				EventType:oData.Response.MediaFileList[iIndex].EventTyp,
		        				MediaFileGUID:oData.Response.MediaFileList[iIndex].MediaFileGUID,
		        				MediaFileName:oData.Response.MediaFileList[iIndex].MediaFileName,
		        				URLPath:oData.Response.MediaFileList[iIndex].URLPath
		        		};
		        		
		        		mediaDefaultConfig.push(oDataObject);
		        	}
		        }
		      }
		      else 
		        lInfo.logln("fetchMediaDefaultConfiguration empty return");
		      
		      callback(mediaDefaultConfig);
		    
		    });
		    
		  } catch (e) {
		    lWarn.logln('fetchMediaDefaultConfiguration error ' + e);
		  }
		  return;
}

/**
 * @brief unregisters trigger for Activity. We have special scenario for
 *        security event when we must also unregister undo triggger
 * 
 * @param oNode
 *            [in] node of Activity
 */
function unRegisterActivityTrigger(oNode) {
	if (oNode != null) {
		var sPath = oNode.getPath();

		lInfo.logln("unregisterTrigger trigger started -> " + sPath);
		unregisterTrigger(sPath);

		// for security events we unregister also undo trigger
		var pUndoTrigger = oNode.getChild('undoCallScene');
		if (pUndoTrigger != null) {
			unregisterTrigger(sPath + "/undoCallScene");
		}
		lInfo.logln("Registering trigger finished");
	}
}

/**
 * @brief registers trigger for Activity. We have special scenario for security
 *        event when we must also register undo triggger
 * 
 * @param oNode
 *            [in] node of Activity
 * @param sTriggerType
 *            [in] Name of trigger type
 * 
 */
function registerActivityTrigger(oNode, sTriggerType) {
	if (oNode != null) {
		var sPath = oNode.getPath();

		lInfo.logln("Registering trigger started -> " + sPath);
		registerTrigger(sPath, sTriggerType, {});

		// for security evente we register also undo trigger
		var pUndoTrigger = oNode.getChild('undoCallScene');
		if (pUndoTrigger != null) {
			registerTrigger(sPath + "/undoCallScene", "sonos_execute_undo", {});
		}
		lInfo.logln("Registering trigger finished");
	}
}

/**
 * Store a boolean property parameter. convert value if possible. In case In
 * case conversion is not possible the value is discarded.
 * 
 * @param path
 *            [in] property path
 * @param value
 *            [in] value to store should be a boolean
 * @returns {Boolean} returns false if value could not be converted
 */
function checkAndStoreBoolParameter(path, value) {
	try {
		lInfo.logln('store boolean at |' + path + "| value: |" + value + "|");
		var successCode = false;
		var boolValue = false;
		if (typeof (value) === "boolean") {
			Property.setProperty(path, value);
			Property.setFlag(path, 'ARCHIVE', true);
			successCode = true;
		} else {
			if (typeof (value) === "string") {
				if (value === "true") {
					boolValue = true;
				}
				Property.setProperty(path, boolValue);
				Property.setFlag(path, 'ARCHIVE', true);
				successCode = true;
			} else {
				lWarn.logln("Typ-Fehler beim Speichern der Property entries/" + path);
			}
		}
		return successCode;
	} catch (e) {
		lWarn.logln("Exception beim checkAndStoreBoolParameter " + e + " Path " + path + " value " + value);
		return false;
	}
}

/**
 * Store a string property parameter. check weather the provided value is a
 * string. In case it's a number it is converted to string, in any other cases
 * it is discarded
 * 
 * @param path
 *            [in] property path
 * @param value
 *            [in] value to store should be string or at least a number
 * @returns {Boolean} returns false if value could not be converted
 */
function checkAndStoreStringParameter(path, value) {
	try {
		lInfo.logln('store string at |' + path + "| value: |" + value + "|");
		var sucessCode = false;
		var stringValue = (typeof (value) === "number") ? String(value) : value;
		if (typeof (stringValue) === "string") {
			Property.setProperty(path, stringValue);
			Property.setFlag(path, 'ARCHIVE', true);
			sucessCode = true;
		} else {
			lWarn.logln("Typ-Fehler beim Speichern der Property entries/" + path);
		}
		return sucessCode;
	} catch (e) {
		lWarn.logln("Exception beim checkAndStoreStringParameter " + e + " Path " + path + " value " + value);
		return false;
	}
}

/**
 * Store a integer property parameter. converts provided value whenever possible
 * to an integer. Discard the the value if this is not possible.
 * 
 * @param path
 *            [in] property path
 * @param value
 *            [in] value to store should be integer
 * @returns {Boolean} returns false if value could not be converted to integer
 */
function checkAndStoreIntParameter(path, value) {
	try {
		lInfo.logln('store integer at |' + path + "| value: |" + value + "|");
		var sucessCode = false;
		var intValue = parseInt("" + value);
		if (!isNaN(intValue)) {
			Property.setProperty(path, intValue);
			Property.setFlag(path, 'ARCHIVE', true);
			sucessCode = true;
		} else {
			lWarn.logln("Typ-Fehler beim Speichern der Property entries/" + path);
		}
		return sucessCode;
	} catch (e) {
		lWarn.logln("Exception beim checkAndStoreIntParameter " + e + " Path " + path + " value " + value);
		return false;
	}
}

/**
 * Stores trigger component data to property tree converts provided value
 * whenever possible to an integer. Discard the the value if this is not
 * possible.
 * 
 * @param idValue
 *            [in] id of activity
 * @param oParameterObject
 *            [in] trigger component data
 * @returns {Boolean} returns true if succeed
 */
function putTriggerParameter(idValue, oParameterObject) {
	var iIndex;
	var triggerIndexCounter = 0;

	for (iIndex = 0; iIndex < oParameterObject.triggers.length; iIndex++) {
		lInfo.logln(JSON.stringify(oParameterObject.triggers[iIndex]));
		if (!oParameterObject.triggers[iIndex].bSkip) {
			var sSubpath = 'sonos-activities/' + idValue + '/triggers/' + (triggerIndexCounter + 1) + "/";
			switch (oParameterObject.triggers[iIndex].type) {
			case 'zone-scene':
			case 'bus-zone-scene':
				// normal zone scene trigger
				checkAndStoreStringParameter(sSubpath + "type", 'bus-zone-scene');
				checkAndStoreIntParameter(sSubpath + "zone", oParameterObject.triggers[iIndex].zone);
				checkAndStoreIntParameter(sSubpath + "group", oParameterObject.triggers[iIndex].group);
				checkAndStoreIntParameter(sSubpath + "scene", oParameterObject.triggers[iIndex].scene);
				if (oParameterObject.triggers[iIndex].forced != null)
					checkAndStoreBoolParameter(sSubpath + "forced", oParameterObject.triggers[iIndex].forced);
				storeSubParameterFull(sSubpath + "dsuid", -1);

				if (predefinedUnduEvents.indexOf(oParameterObject.triggers[iIndex].scene) != -1) {
					// we create additional trigger for UNDO scene
					sSubpath = 'sonos-activities/' + idValue + '/undoCallScene/triggers/' + (triggerIndexCounter + 1) + "/";
					checkAndStoreStringParameter(sSubpath + "type", 'undo-zone-scene');
					checkAndStoreIntParameter(sSubpath + "zone", oParameterObject.triggers[iIndex].zone);
					checkAndStoreIntParameter(sSubpath + "group", oParameterObject.triggers[iIndex].group);
					checkAndStoreIntParameter(sSubpath + "scene", oParameterObject.triggers[iIndex].scene);
					storeSubParameterFull(sSubpath + "dsuid", -1);
				}
				break;
			case 'undo-zone-scene':
				checkAndStoreStringParameter(sSubpath + "type", oParameterObject.triggers[iIndex].type);
				checkAndStoreIntParameter(sSubpath + "zone", oParameterObject.triggers[iIndex].zone);
				checkAndStoreIntParameter(sSubpath + "group", oParameterObject.triggers[iIndex].group);
				checkAndStoreIntParameter(sSubpath + "scene", oParameterObject.triggers[iIndex].scene);
				storeSubParameterFull(sSubpath + "dsuid", -1);
				break;
			case 'device-msg':
				checkAndStoreStringParameter(sSubpath + "type", oParameterObject.triggers[iIndex].type);
				checkAndStoreDSIDParameter(sSubpath + "dsuid", oParameterObject.triggers[iIndex].dsuid, oParameterObject.triggers[iIndex].dsid);
                checkAndStoreIntParameter(sSubpath + "msg", oParameterObject.triggers[iIndex].msg);
				if (oParameterObject.triggers[iIndex].buttonIndex != null)
					checkAndStoreIntParameter(sSubpath + "buttonIndex", oParameterObject.triggers[iIndex].buttonIndex);
				break;
			case 'device-binary-input':
				checkAndStoreStringParameter(sSubpath + "type", oParameterObject.triggers[iIndex].type);
				checkAndStoreDSIDParameter(sSubpath + "dsuid", oParameterObject.triggers[iIndex].dsuid, oParameterObject.triggers[iIndex].dsid);
				checkAndStoreIntParameter(sSubpath + "index", oParameterObject.triggers[iIndex].index);
				checkAndStoreStringParameter(sSubpath + "state", oParameterObject.triggers[iIndex].state);
				break;
			case 'device-scene':
				checkAndStoreStringParameter(sSubpath + "type", oParameterObject.triggers[iIndex].type);
				checkAndStoreDSIDParameter(sSubpath + "dsuid", oParameterObject.triggers[iIndex].dsuid, oParameterObject.triggers[iIndex].dsid);
				checkAndStoreIntParameter(sSubpath + "scene", oParameterObject.triggers[iIndex].scene);
				break;
			case 'custom-event':
				checkAndStoreStringParameter(sSubpath + "type", oParameterObject.triggers[iIndex].type);
				checkAndStoreStringParameter(sSubpath + "event", ""+oParameterObject.triggers[iIndex].event);
				break;
			case 'device-sensor':
				checkAndStoreStringParameter(sSubpath + "type", oParameterObject.triggers[iIndex].type);
				checkAndStoreDSIDParameter(sSubpath + "dsuid", oParameterObject.triggers[iIndex].dsuid, oParameterObject.triggers[iIndex].dsid);
				checkAndStoreStringParameter(sSubpath + "eventid", oParameterObject.triggers[iIndex].eventid);
				break;
			case 'state-change':
				checkAndStoreStringParameter(sSubpath + "type", oParameterObject.triggers[iIndex].type);
				checkAndStoreStringParameter(sSubpath + "name", oParameterObject.triggers[iIndex].name);
				checkAndStoreStringParameter(sSubpath + "state", oParameterObject.triggers[iIndex].state);

				if (oParameterObject.triggers[iIndex].name == "fire") {
					sSubpath = 'sonos-activities/' + idValue + '/undoCallScene/triggers/' + (triggerIndexCounter + 1) + "/";
					checkAndStoreStringParameter(sSubpath + "type", oParameterObject.triggers[iIndex].type);
					checkAndStoreStringParameter(sSubpath + "name", oParameterObject.triggers[iIndex].name);
					checkAndStoreStringParameter(sSubpath + "state", "inactive");
				}
				break;
			case 'addon-state-change':
				checkAndStoreStringParameter(sSubpath + "type", oParameterObject.triggers[iIndex].type);
				checkAndStoreStringParameter(sSubpath + "addon-id", oParameterObject.triggers[iIndex].addonId);
				checkAndStoreStringParameter(sSubpath + "name", oParameterObject.triggers[iIndex].name);
				checkAndStoreStringParameter(sSubpath + "state", oParameterObject.triggers[iIndex].state);
				break;
			}
			triggerIndexCounter++;
		}
	}
}

/**
 * @brief Creates new ID required for storing data to PropertyTree
 * 
 * @param sNode
 *            [in] node from whitch to get new ID
 * 
 * @return New ID value
 */
function getNewIdValue(sNode) {
	var idValue = 0;
	var baseNode = Property.getNode(sNode);
	if (baseNode == null)
		return idValue;
	var oArrayNode = baseNode.getChildren();
	if (oArrayNode != null) {
		for ( var iIndex = 0; iIndex < oArrayNode.length; iIndex++) {
			var oChildNode = oArrayNode[iIndex];
			if (oChildNode.getChild('id') != null) {
				if (oChildNode.getChild('id').getValue() != null)
					if (parseInt("" + oChildNode.getChild('id').getValue(), 10) >= idValue)
						idValue = parseInt("" + oChildNode.getChild('id').getValue(), 10) + 1;
			}
		}
	}
	return idValue;
}

/**
 * @brief Searching for player in PropertyTree
 * 
 * @param sMasterPlayerID
 *            [in] main playerID
 * @param sSlavePlayers
 *            [in] list of sub players separated with comma
 * 
 * @return Id of searched player
 */
function getDefaultDeviceID(sMasterPlayerID, sSlavePlayers) {
	var idValue = -1;

	var arrayPlayers = new Array(); // list of all players in sonos group
	arrayPlayers.push(sMasterPlayerID);

	var bHasSlaves = false;
	if (sSlavePlayers.length > 0) {
		var aSlaves = sSlavePlayers.split(",");
		if (aSlaves != null && aSlaves.length > 0) {
			for ( var iIndex = 0; iIndex < aSlaves.length; iIndex++) {
				if (aSlaves[iIndex].trim().length > 0) {
					arrayPlayers.push(aSlaves[iIndex].trim());
					bHasSlaves = true;
				}
			}
		}
	}

	lInfo.logln("arrayPlayers=" + JSON.stringify(arrayPlayers));

	var baseNode = Property.getNode('sonos-players');
	if (baseNode != null) {
		var oArrayNode = baseNode.getChildren();
		for ( var iIndex = 0; iIndex < oArrayNode.length; iIndex++) {
			var oChildNode = oArrayNode[iIndex];
			if (oChildNode.getChild('deviceID') != null) {
				if (arrayPlayers.indexOf(oChildNode.getChild('deviceID').getValue()) > -1) {
					if (bHasSlaves) {
						if (oChildNode.getChild('playersListSlaves') != null && oChildNode.getChild('playersListSlaves').getValue().length > 0) {
							var aStoredSlaves = oChildNode.getChild('playersListSlaves').getValue().split(",");
							if (aStoredSlaves != null && aStoredSlaves.length == (arrayPlayers.length - 1)) {
								bAllSubFound = true;
								for ( var yIndex = 0; yIndex < aStoredSlaves.length; yIndex++) {
									if (arrayPlayers.indexOf(aStoredSlaves[yIndex].trim()) < 0) {
										lInfo.logln("not found player*" + aStoredSlaves[yIndex].trim() + "*");
										bAllSubFound = false;
										break;
									}
								}
								if (bAllSubFound) {
									idValue = oChildNode.getChild('id').getValue();
									break;
								}
							}
						}
					} else {
						if (sMasterPlayerID == oChildNode.getChild('deviceID').getValue() && oChildNode.getChild('playersListSlaves') != null
								&& oChildNode.getChild('playersListSlaves').getValue().length == 0) {
							idValue = oChildNode.getChild('id').getValue();
							break;
						}
					}
				}
			}
		}
	}
	return idValue;
}

/**
 * @brief Searching for preset action in PropertyTree
 * 
 * @param sPlayerID
 *            [in] id of player
 * @param nPresetID
 *            [in] seacrhed preset ID
 * 
 * @return Id of searched preset action, or -1 if not found
 */
function getDefaultPresetID(nRoomID, nPresetID) {
	var idValue = -1;
	var baseNode = Property.getNode('sonos-presets');
	if (baseNode != null) {
		var oArrayNode = baseNode.getChildren();
		for ( var iIndex = 0; iIndex < oArrayNode.length; iIndex++) {
			var oChildNode = oArrayNode[iIndex];
			if (oChildNode.getChild('roomID') != null) {
				if (oChildNode.getChild('roomID').getValue() != null && oChildNode.getChild('roomID').getValue() == nRoomID) {
					if (oChildNode.getChild('presetID') != null) {
						if (oChildNode.getChild('presetID').getValue() != null && oChildNode.getChild('presetID').getValue() == nPresetID) {
							if (oChildNode.getChild('id').getValue() != null) {
								idValue = oChildNode.getChild('id').getValue()
								break;
							}
						}
					}
				}
			}
		}
	}
	return idValue;
}

/**
 * @brief Searching dSS room from list with sonos group
 * 
 * @param roomsList
 *            [in] list of rooms
 * @param sSonosGroup
 *            [in] sonos groupID
 * 
 * @return Id of dSS room
 */
function getStoredRoom(roomsList, sSonosGroup) {
	var nRoomID = 0;
	for ( var iIndex = 0; iIndex < roomsList.length; iIndex++) {
		var oChildNode = roomsList[iIndex];
		if (oChildNode.groupID == sSonosGroup) {
			nRoomID = oChildNode.roomID;
			break;
		}
	}
	return nRoomID;
}

function clearSonosGrouping() {
	var baseNode = Property.getNode('sonos-groups');
	if (baseNode != null) {
		var oArrayNode = baseNode.getChildren();
		for ( var iIndex = 0; iIndex < oArrayNode.length; iIndex++) {
			var oChildNode = oArrayNode[iIndex];
			if (oChildNode != null)
				baseNode.removeChild(oChildNode.getChild('id').getValue());
		}
	}
}

/**
 * @brief Storing Sonos Playlist to PropertyTree
 * 
 * @param arrayList
 *            [in] list of Sonos Playlist
 * 
 * @return
 */
function saveSonosPlayList(arrayList) {
	try {
		lInfo.logln("saveSonosPlayList1");
		// Delete old list
		var baseNode = Property.getNode('sonos-playlists');
		if (baseNode != null) {
			var oArrayNode = baseNode.getChildren();
			for ( var iIndex = 0; iIndex < oArrayNode.length; iIndex++) {
				var oChildNode = oArrayNode[iIndex];
				baseNode.removeChild(oChildNode.getChild('id').getValue());
			}
		}

		if (arrayList != null) {
			for ( var iIndex = 0; iIndex < arrayList.length; iIndex++) {
				var sData = arrayList[iIndex].data;
				Property.setProperty('sonos-playlists/' + iIndex + '/id', iIndex);
				Property.setProperty('sonos-playlists/' + iIndex + '/title', unescape(htmlUnescape(arrayList[iIndex].title)));
				Property.setProperty('sonos-playlists/' + iIndex + '/listURL', unescape(arrayList[iIndex].listCall));
				Property.setProperty('sonos-playlists/' + iIndex + '/playListSonosID', unescape(arrayList[iIndex].listID));
			}
		}

	} catch (e) {
		lInfo.logln("Error by saving saveSonosPlayList: " + e);
		for ( var i in e) {
			lInfo.logln(i + ': ' + e[i]);
		}
	}
}

/**
 * @brief Storing Sonos Favorites to PropertyTree
 * 
 * @param arrayList
 *            [in] list of Sonos Favorites
 * 
 * @return
 */
function saveSonosFavorites(arrayList) {
	try {
		// Delete old list
		var baseNode = Property.getNode('sonos-favorites');
		if (baseNode != null) {
			var oArrayNode = baseNode.getChildren();
			for ( var iIndex = 0; iIndex < oArrayNode.length; iIndex++) {
				var oChildNode = oArrayNode[iIndex];
				baseNode.removeChild(oChildNode.getChild('id').getValue());
			}
		}

		if (arrayList != null) {
			for ( var iIndex = 0; iIndex < arrayList.length; iIndex++) {
				var sData = arrayList[iIndex].data;
				Property.setProperty('sonos-favorites/' + iIndex + '/id', iIndex);
				Property.setProperty('sonos-favorites/' + iIndex + '/title', unescape(htmlUnescape(arrayList[iIndex].title)));
				Property.setProperty('sonos-favorites/' + iIndex + '/description', unescape(htmlUnescape(arrayList[iIndex].description)));
				Property.setProperty('sonos-favorites/' + iIndex + '/listURL', (arrayList[iIndex].listCall));
				Property.setProperty('sonos-favorites/' + iIndex + '/metaData', (arrayList[iIndex].metaData));
				Property.setProperty('sonos-favorites/' + iIndex + '/playListSonosID', unescape(arrayList[iIndex].listID));
				Property.setProperty('sonos-favorites/' + iIndex + '/isSingleTrack', JSON.parse(arrayList[iIndex].isSingleTrack));
			}
		}
	} catch (e) {
		lInfo.logln("Error by saving saveSonosFavorites: " + e);
		for ( var i in e) {
			lInfo.logln(i + ': ' + e[i]);
		}
	}
}

/**
 * @brief Stores Sonos Player data to PropertyTree
 * 
 * @param objPlayer
 *            [in] - Object with Sonos player data
 * 
 * @return
 */
function storePlayer(objPlayer, arrayAssignedRooms) {
	try {
		var playerPropertyID = getDefaultDeviceID(objPlayer.playerID, objPlayer.playersInGroup);
		var bNewDevice = false;

		if (playerPropertyID < 0) {
			playerPropertyID = getNewIdValue('sonos-players');
			bNewDevice = true;

			var nRoomID = 0;
			if (arrayAssignedRooms != null && arrayAssignedRooms.length > 0) {
				for (iIndex = 0; iIndex < arrayAssignedRooms.length; iIndex++) {
					if (arrayAssignedRooms[iIndex].deviceID == unescape(objPlayer.playerID)) {
						nRoomID = arrayAssignedRooms[iIndex].roomID;
						break;
					}
				}
			}

			storeSubParameter('sonos-players', playerPropertyID + '/roomID', nRoomID);
		}

		// storing data to propertyTree
		storeSubParameter('sonos-players', playerPropertyID + '/id', playerPropertyID);
		storeSubParameter('sonos-players', playerPropertyID + '/ipAddress', unescape(objPlayer.deviceIP));
		storeSubParameter('sonos-players', playerPropertyID + '/sonosRoom', unescape(objPlayer.sonosRoom));
		storeSubParameter('sonos-players', playerPropertyID + '/playersList', unescape(objPlayer.playerName));
		storeSubParameter('sonos-players', playerPropertyID + '/playersListSerial', unescape(objPlayer.serialNum));
		storeSubParameter('sonos-players', playerPropertyID + '/playersListSlaves', unescape(objPlayer.playersInGroup));
		storeSubParameter('sonos-players', playerPropertyID + '/playersListIPSlaves', unescape(objPlayer.playersInGroupIP));
		storeSubParameter('sonos-players', playerPropertyID + '/sonosGroup', unescape(objPlayer.sonosGroup));
		storeSubParameter('sonos-players', playerPropertyID + '/disabled', 'false');
		storeSubParameter('sonos-players', playerPropertyID + '/notDiscovered', 'false');
		storeSubParameter('sonos-players', playerPropertyID + '/deviceID', unescape(objPlayer.playerID));

		fShouldStore = true;

		return playerPropertyID;
	} catch (e) {
		lInfo.logln("Error by saving storePlayer: " + e);
		for ( var i in e) {
			lInfo.logln(i + ': ' + e[i]);
		}
	}
	return INVALID_PLAYER_ID;
}

/**
 * @brief Stores Preset action to PropertyTree
 * 
 * @param objPreset
 *            [in] - Object with Preset action data
 * 
 * @return
 */
function storePreset(objPreset) {
	try {
		var nPresetID = parseInt(objPreset.triggerPreset.presetID, 10);

		var presetPropertyID = objPreset.id;
		var bNewDevice = false;

		if (presetPropertyID == null) {
			presetPropertyID = getNewIdValue('sonos-presets');
			bNewDevice = true;
		}

		var nAction = parseInt("" + objPreset.whatToPlay.actionType, 10);
		var nRoomID = parseInt("" + (objPreset.triggerPreset.roomID), 10);
		if (Property.getNode('sonos-presets/' + presetPropertyID) != null)
			Property.getNode('sonos-presets').removeChild("" + presetPropertyID);
		// storing data to propertyTree
		storeSubParameter('sonos-presets', presetPropertyID + '/audioName', '');
		storeSubParameter('sonos-presets', presetPropertyID + '/id', presetPropertyID);
		storeSubParameter('sonos-presets', presetPropertyID + '/roomID', nRoomID);
		storeSubParameter('sonos-presets', presetPropertyID + '/presetID', nPresetID);
		// storeSubParameter('sonos-presets', presetPropertyID+'/presetName',
		// unescape(objPreset.triggerPreset.presetName));
		storeSubParameter('sonos-presets', presetPropertyID + '/volumeSet', unescape(objPreset.whatToPlay.setVolume));
		storeSubParameter('sonos-presets', presetPropertyID + '/volume', unescape(objPreset.whatToPlay.volume));
		storeSubParameter('sonos-presets', presetPropertyID + '/actionType', nAction);
		storeSubParameter('sonos-presets', presetPropertyID + '/disabled', 'false');
		storeSubParameter('sonos-presets', presetPropertyID + '/conditions/enabled', true);


		switch (nAction) {
		case 0: // now playing

			var nowAction = parseInt("" + objPreset.whatToPlay.nowAction, 10);
			if (objPreset.whatToPlay.selectedData.changed != null) {
				var bSave = JSON.parse(objPreset.whatToPlay.selectedData.changed);
				storeSubParameter('sonos-presets', presetPropertyID + '/nowAction', nowAction);
				if (bSave) {
					if (nowAction == 0) { // single song/radio
						storeSubParameter('sonos-presets', presetPropertyID + '/nowSingleTitle', Utf8.encode(unescape(objPreset.whatToPlay.selectedData.title)));
						storeSubParameter('sonos-presets', presetPropertyID + '/nowSingleIsRadio', objPreset.whatToPlay.selectedData.isRadio);
						storeSubParameter('sonos-presets', presetPropertyID + '/nowSingleArtist', Utf8.encode(unescape(objPreset.whatToPlay.selectedData.artist)));
						storeSubParameter('sonos-presets', presetPropertyID + '/nowSingleUrl', unescape(objPreset.whatToPlay.selectedData.url));
					} else { // active queue
						if (!bNewDevice) { // delete old values
							var baseNode = Property.getNode('sonos-presets/' + presetPropertyID + '/active-queue');
							if (baseNode != null) {
								var oArrayNode = baseNode.getChildren();
								for ( var iIndex = 0; iIndex < oArrayNode.length; iIndex++) {
									var oChildNode = oArrayNode[iIndex];
									baseNode.removeChild(oChildNode.getChild('id').getValue());
								}
							}
						}

						var baseNode = Property.getNode('sonos-selected-preset');
						var queue = null;
						if (baseNode != null) {
							queue = JSON.parse(unescape(baseNode.getChild('tmp-queue').getValue()));
							for ( var iIndex = 0; iIndex < queue.length; iIndex++) {
								var oQueueItem = queue[iIndex];
								storeSubParameter('sonos-presets', presetPropertyID + '/active-queue/' + iIndex + "/id", iIndex);
								storeSubParameter('sonos-presets', presetPropertyID + '/active-queue/' + iIndex + "/title", Utf8.encode(unescape(oQueueItem.title)));
								storeSubParameter('sonos-presets', presetPropertyID + '/active-queue/' + iIndex + "/playID", unescape(oQueueItem.playID));
								// storeSubParameter('sonos-presets',
								// presetPropertyID+'/active-queue/'+iIndex+"/albumUri",
								// unescape(oQueueItem.albumUri));
								storeSubParameter('sonos-presets', presetPropertyID + '/active-queue/' + iIndex + "/artist", Utf8.encode(unescape(oQueueItem.artist)));
								storeSubParameter('sonos-presets', presetPropertyID + '/active-queue/' + iIndex + "/trackPos", unescape(oQueueItem.trackPos));
							}
						}
					}
				}
			}
			break;

		case 1: // playlist

			storeSubParameter('sonos-presets', presetPropertyID + '/playListTitle', Utf8.encode(unescape(objPreset.whatToPlay.title)));
			storeSubParameter('sonos-presets', presetPropertyID + '/playListURL', unescape(objPreset.whatToPlay.listUrl));
			storeSubParameter('sonos-presets', presetPropertyID + '/playListSonosID', unescape(objPreset.whatToPlay.playListSonosID));
			break;

		case 2: // favorites

			storeSubParameter('sonos-presets', presetPropertyID + '/favoritesTitle', Utf8.encode(unescape(objPreset.whatToPlay.title)));
			storeSubParameter('sonos-presets', presetPropertyID + '/favoritesURL', objPreset.whatToPlay.listUrl);
			storeSubParameter('sonos-presets', presetPropertyID + '/metaData', objPreset.whatToPlay.metaData);
			storeSubParameter('sonos-presets', presetPropertyID + '/favoritesSonosID', unescape(objPreset.whatToPlay.playListSonosID));
			break;
		}

		/* Storing trigger structure */
		// Delete old trigger
		var baseNode = Property.getNode('sonos-presets/' + presetPropertyID + '/triggers');
		if (baseNode != null) {
			var oArrayNode = baseNode.getChildren();
			for ( var iIndex = 0; iIndex < oArrayNode.length; iIndex++) {
				baseNode.removeChild((iIndex + 1));
			}
		}

		// add new trigger
		for ( var iX = 1; iX <= 4; iX++) {
			storeSubParameter('sonos-presets', presetPropertyID + '/triggers/' + iX + '/type', 'bus-zone-scene');
			storeSubParameter('sonos-presets', presetPropertyID + '/triggers/' + iX + '/scene', nPresetID);
			storeSubParameter('sonos-presets', presetPropertyID + '/triggers/' + iX + '/dsuid', -1);

			var nTrgGroupID = 4;
			var nTrgZone = nRoomID;
			switch (iX) {
			case 1:
				nTrgGroupID = 4;
				nTrgZone = nRoomID;
				break;

			case 2:
				nTrgGroupID = 4;
				nTrgZone = 0;
				break;

			case 3:
				nTrgGroupID = 0;
				nTrgZone = nRoomID;
				break;

			case 4:
				nTrgGroupID = 0;
				nTrgZone = 0;
				break;
			}
			storeSubParameter('sonos-presets', presetPropertyID + '/triggers/' + iX + '/group', nTrgGroupID);
			storeSubParameter('sonos-presets', presetPropertyID + '/triggers/' + iX + '/zone', nTrgZone);
		}
		storeSubParameter('sonos-selected-preset', 'tmp-queue', "");

		var sPath = Property.getNode('sonos-presets/' + presetPropertyID).getPath();
		unregisterTrigger(sPath);
		registerTrigger(sPath, 'sonos_execute', {});
		lInfo.logln("RegTrigger: " + sPath);

		storeSubParameter('sonos-presets', presetPropertyID + '/audioName', Utf8.encode(unescape(objPreset.audioName)));

		fShouldStore = true;
	} catch (e) {
		lInfo.logln("Error by saving storePreset: " + e);
		for ( var i in e) {
			lInfo.logln(i + ': ' + e[i]);
		}
	}
}

/**
 * @brief Stores Activity action to PropertyTree
 * 
 * @param objPreset
 *            [in] - Object with Activity action data
 * 
 * @return
 */
function storeActivity(objPreset, bExecutePropertyStore) {
	try {
		var sResultInfo = "";
		var activityPropertyID = objPreset.id;
		lInfo.logln("activityPropertyID=" + objPreset.id);
		var bNewDevice = false;
		if (activityPropertyID == null) {
			activityPropertyID = getNewIdValue('sonos-activities');
			bNewDevice = true;
		}
		if (Property.getNode('sonos-activities/' + activityPropertyID) != null) {
			unRegisterActivityTrigger(Property.getNode('sonos-activities/' + activityPropertyID));
			var undoTriggerNode = Property.getNode('sonos-activities/' + activityPropertyID + '/undoCallScene/triggers');
			if (undoTriggerNode != null) {
				Property.getNode('sonos-activities/' + activityPropertyID).removeChild("undoCallScene");
			}
			Property.getNode('sonos-activities').removeChild("" + activityPropertyID);
		}

		putTriggerParameter(activityPropertyID, objPreset);

		var nAction = parseInt("" + objPreset.whatToPlay.actionType, 10);

		// storing data to propertyTree
		storeSubParameter('sonos-activities', activityPropertyID + '/id', activityPropertyID);
		storeSubParameter('sonos-activities', activityPropertyID + '/volumeSet', unescape(objPreset.whatToPlay.setVolume));
		storeSubParameter('sonos-activities', activityPropertyID + '/volume', unescape(objPreset.whatToPlay.volume));
		storeSubParameter('sonos-activities', activityPropertyID + '/setRepeat', unescape(objPreset.whatToPlay.setRepeat));
		storeSubParameter('sonos-activities', activityPropertyID + '/repeatTime', unescape(objPreset.whatToPlay.repeatTime));
		storeSubParameter('sonos-activities', activityPropertyID + '/restartAfter', unescape(objPreset.whatToPlay.restartAfter));
		storeSubParameter('sonos-activities', activityPropertyID + '/createSonosGroup', unescape(objPreset.selectedRoom.createSonosGroup));
		storeSubParameter('sonos-activities', activityPropertyID + '/actionType', nAction);
		storeSubParameter('sonos-activities', activityPropertyID + '/disabled', 'false');
		storeSubParameter('sonos-activities', activityPropertyID + '/conditions/enabled', true);

		if (objPreset.isPredefined != null)
			storeSubParameter('sonos-activities', activityPropertyID + '/predefined', 'true');

		// Where to play
		if (objPreset.selectedRoom.action == 0) // all rooms
			storeSubParameter('sonos-activities', activityPropertyID + '/allRooms', 1);
		else {
			var sRooms = "";
			for ( var iIndex = 0; iIndex < objPreset.selectedRoom.rooms.length; iIndex++) {
				if (iIndex > 0)
					sRooms += ";"
				sRooms += objPreset.selectedRoom.rooms[iIndex];
			}

			storeSubParameter('sonos-activities', activityPropertyID + '/allRooms', 0);
			storeSubParameter('sonos-activities', activityPropertyID + '/pickedRooms', sRooms);
		}
		storeSubParameter('sonos-selected-preset', 'tmp-queue', "");

		// What to play
		switch (nAction) {
		case SONOS_ACTION_NOW_PLAYING: // now playing

			var nowAction = parseInt("" + objPreset.whatToPlay.nowAction, 10);
			storeSubParameter('sonos-activities', activityPropertyID + '/nowAction', nowAction);
			if (objPreset.whatToPlay.selectedData.changed != null) {
				var bSave = JSON.parse(objPreset.whatToPlay.selectedData.changed);
				if (bSave) {
					if (nowAction == 0) { // single song/radio
						storeSubParameter('sonos-activities', activityPropertyID + '/nowSingleTitle', Utf8.encode(unescape(objPreset.whatToPlay.selectedData.title)));
						storeSubParameter('sonos-activities', activityPropertyID + '/nowSingleIsRadio', unescape(objPreset.whatToPlay.selectedData.isRadio));
						storeSubParameter('sonos-activities', activityPropertyID + '/nowSingleArtist', Utf8.encode(unescape(objPreset.whatToPlay.selectedData.artist)));
						storeSubParameter('sonos-activities', activityPropertyID + '/nowSingleUrl', unescape(objPreset.whatToPlay.selectedData.url));
						if (objPreset.whatToPlay.selectedData.length != null && objPreset.whatToPlay.selectedData.length != "") {
							var nLength = secondToInt(unescape(objPreset.whatToPlay.selectedData.length));
							if (!isNaN(nLength))
								storeSubParameter('sonos-activities', activityPropertyID + '/nowSingleLength', nLength);
						}
					} else { // active queue
						if (!bNewDevice) { // delete old values
							var baseNode = Property.getNode('sonos-activities/' + activityPropertyID + '/active-queue');
							if (baseNode != null) {
								var oArrayNode = baseNode.getChildren();
								for ( var iIndex = 0; iIndex < oArrayNode.length; iIndex++) {
									var oChildNode = oArrayNode[iIndex];
									baseNode.removeChild(oChildNode.getChild('id').getValue());
								}
							}
						}

						var baseNode = Property.getNode('sonos-selected-preset');
						var queue = null;
						if (baseNode != null) {
							queue = JSON.parse(unescape(baseNode.getChild('tmp-queue').getValue()));
							for ( var iIndex = 0; iIndex < queue.length; iIndex++) {
								var oQueueItem = queue[iIndex];
								storeSubParameter('sonos-activities', activityPropertyID + '/active-queue/' + iIndex + "/id", iIndex);
								storeSubParameter('sonos-activities', activityPropertyID + '/active-queue/' + iIndex + "/title", Utf8.encode(unescape(oQueueItem.title)));
								storeSubParameter('sonos-activities', activityPropertyID + '/active-queue/' + iIndex + "/playID", unescape(oQueueItem.playID));
								// storeSubParameter('sonos-activities',
								// activityPropertyID+'/active-queue/'+iIndex+"/albumUri",
								// unescape(oQueueItem.albumUri));
								storeSubParameter('sonos-activities', activityPropertyID + '/active-queue/' + iIndex + "/artist", Utf8.encode(unescape(oQueueItem.artist)));
								storeSubParameter('sonos-activities', activityPropertyID + '/active-queue/' + iIndex + "/trackPos", unescape(oQueueItem.trackPos));
							}
						}
					}
				}
			}
			break;

		case SONOS_ACTION_PLAYLIST: // playlist

			storeSubParameter('sonos-activities', activityPropertyID + '/playListTitle', Utf8.encode(unescape(objPreset.whatToPlay.title)));
			storeSubParameter('sonos-activities', activityPropertyID + '/playListURL', unescape(objPreset.whatToPlay.listUrl));
			storeSubParameter('sonos-activities', activityPropertyID + '/playListSonosID', unescape(objPreset.whatToPlay.playListSonosID));
			break;

		case SONOS_ACTION_FAVORITES: // favorites

			storeSubParameter('sonos-activities', activityPropertyID + '/favoritesTitle', Utf8.encode(unescape(objPreset.whatToPlay.title)));
			storeSubParameter('sonos-activities', activityPropertyID + '/favoritesURL', objPreset.whatToPlay.listUrl);
			storeSubParameter('sonos-activities', activityPropertyID + '/metaData', objPreset.whatToPlay.metaData);
			storeSubParameter('sonos-activities', activityPropertyID + '/favoritesSonosID', unescape(objPreset.whatToPlay.playListSonosID));
			break;

		case SONOS_ACTION_LIBRARY:

			var oldGuidNode = Property.getNode('sonos-activities/' + activityPropertyID + '/mediaFileGUID');
			var sOldGuidID = "";
			if (oldGuidNode != null)
				sOldGuidID = oldGuidNode.getValue();

			if (objPreset.whatToPlay.mediaFileGUID != sOldGuidID) { // new guid
				// is
				// different,
				// we update
				// data and
				// fetch url
				// file
				storeSubParameter('sonos-activities', activityPropertyID + '/mediaFileTitle', Utf8.encode(unescape(objPreset.whatToPlay.title)));
				storeSubParameter('sonos-activities', activityPropertyID + '/mediaGroupGUID', unescape(objPreset.whatToPlay.mediaGroupGUID));
				storeSubParameter('sonos-activities', activityPropertyID + '/mediaGroupSubGUID', unescape(objPreset.whatToPlay.mediaGroupSubGUID));
				storeSubParameter('sonos-activities', activityPropertyID + '/mediaFileGUID', unescape(objPreset.whatToPlay.mediaFileGUID));

				// get media file url
				if (objPreset.whatToPlay.mediaFileGUID != '') {
					cloudService.fetchMediaFile(unescape(objPreset.whatToPlay.mediaFileGUID), function(oData) {
						if (oData.ReturnMessage == "OK") {
							storeSubParameter('sonos-activities', activityPropertyID + '/nowSingleUrl', unescape(oData.Response.UrlPath));
							storeSubParameter('sonos-activities', activityPropertyID + '/nowSingleLength', '');
							(new Event('sonos-controller.storePropertyTree', {})).raise();
						}
					});
				}
			} else
				lInfo.logln("Saving Media Lib info skip, since no changes have been made");

			break;

		case SONOS_ACTION_TTS:

			var t2sText = Utf8.encode(unescape(objPreset.whatToPlay.t2sText));
			var t2sLang = unescape(objPreset.whatToPlay.t2sLangID);
			var t2sVoice = unescape(objPreset.whatToPlay.t2sVoiceID);
			var t2sFolder = unescape(objPreset.whatToPlay.t2sFolder);

			storeSubParameter('sonos-activities', activityPropertyID + '/t2Stext', t2sText);
			storeSubParameter('sonos-activities', activityPropertyID + '/t2SlangID', t2sLang);
			storeSubParameter('sonos-activities', activityPropertyID + '/t2SvoiceID', t2sVoice);

			// if we have cretaed GUID file, we just fetch file url
			if (objPreset.whatToPlay.t2sGUID != '') {
				var oldGuidNode = Property.getNode('sonos-activities/' + activityPropertyID + '/t2Sguid');
				var sOldGuidID = "";
				if (oldGuidNode != null)
					sOldGuidID = oldGuidNode.getValue();

				if (objPreset.whatToPlay.t2sGUID != sOldGuidID) { // new guid
					// is
					// different,
					// we must
					// fecth
					// file url
					cloudService.fetchMediaFile(objPreset.whatToPlay.t2sGUID, function(oData) {
						if (oData.ReturnMessage == "OK") {
							storeSubParameter('sonos-activities', activityPropertyID + '/nowSingleUrl', unescape(oData.Response.UrlPath));
							storeSubParameter('sonos-activities', activityPropertyID + '/nowSingleLength', '');
							storeSubParameter('sonos-activities', activityPropertyID + '/t2Sguid', unescape(objPreset.whatToPlay.t2sGUID));
							(new Event('sonos-controller.storePropertyTree', {})).raise();
						}
					});
				}
			} else { // we don't have GUID, we must first create file and
				// then fetch file url
				cloudService.createTextSound(t2sFolder, t2sLang, t2sVoice, unescape(objPreset.whatToPlay.t2sText), function(oData) {
					if (oData.ReturnMessage == "OK") {
						lInfo.logln("ACT SAVE - createTextSound=" + oData.Response.TextToSpeechVoiceFileGuid);
						if (oData.Response.TextToSpeechVoiceFileGuid != "") {
							var fileGuid = unescape(oData.Response.TextToSpeechVoiceFileGuid);
							cloudService.fetchMediaFile(fileGuid, function(oData) {
								if (oData.ReturnMessage == "OK") {
									lInfo.logln("ACT SAVE - Media url=" + oData.Response.UrlPath);
									if (oData.Response.UrlPath != "") {
										storeSubParameter('sonos-activities', activityPropertyID + '/nowSingleUrl', unescape(oData.Response.UrlPath));
										storeSubParameter('sonos-activities', activityPropertyID + '/t2Sguid', fileGuid);
										storeSubParameter('sonos-activities', activityPropertyID + '/nowSingleLength', '');
										(new Event('sonos-controller.storePropertyTree', {})).raise();
									}
								}
							});
						} else {
							lInfo.logln("Storing failed TextToSpeechVoiceFileGuid");
						}
					} else {
						lInfo.logln("Storing failed createTextSound");
					}
				});
			}

			// we must delete old file
			try {
				lInfo.logln(JSON.stringify(objPreset.whatToPlay));
				if (objPreset.whatToPlay.t2sOriginalGUID != "" && objPreset.whatToPlay.t2sGUID != objPreset.whatToPlay.t2sOriginalGUID) {
					cloudService.deleteTextSound(objPreset.whatToPlay.t2sOriginalGUID, function(oData) {
					});
				}
			} catch (e) {
			}
			break;

		case SONOS_ACTION_COMMAND: // command

			storeSubParameter('sonos-activities', activityPropertyID + '/command', unescape(objPreset.whatToPlay.command));
			break;
		}

		if (bExecutePropertyStore) {
			fShouldStore = true;
		}

		var node = Property.getNode('sonos-activities/' + activityPropertyID);
		registerActivityTrigger(node, 'sonos_execute');
		lInfo.logln("Registering trigger finished");
	} catch (e) {
		lInfo.logln("Error by saving storeActivity: " + e);
		for ( var i in e) {
			lInfo.logln(i + ': ' + e[i]);
		}
	}
	return sResultInfo;
}

/**
 * @brief Function is checking in selected room has allready stored predefined
 *        triggers
 * 
 * @param roomID
 *            [in] - ID of searched room
 * 
 * @return true if triggers are stored, false if room has no default triggers
 *         stored
 */
function testIfRoomHasDefaultTriggers(roomID) {
	var baseNode = Property.getNode('sonos-presets');
	if (baseNode != null) {
		var oArrayNode = baseNode.getChildren();
		for ( var iIndex = 0; iIndex < oArrayNode.length; iIndex++) {
			var oChildNode = oArrayNode[iIndex];
			if (oChildNode.getChild('roomID') != null)
				if (oChildNode.getChild('roomID').getValue() == roomID)
					if (oChildNode.getChild('predefined').getValue())
						return true;
		}
	}
	return false;
}

/**
 * @brief Function creates predefined triggers(predefinedRoomEvents defined in
 *        file defined-values.js) for selected room
 * @param roomID
 *            [in] - ID of room to store triggers
 * 
 * @return
 */
function createDefaultRoomPresets(roomID) {
	if (predefinedRoomEvents != null && predefinedRoomEvents.length > 0) {
		for (index in predefinedRoomEvents) {
			var presetPropertyID = getNewIdValue('sonos-presets');
			storeSubParameter('sonos-presets', presetPropertyID + '/id', presetPropertyID);
			storeSubParameter('sonos-presets', presetPropertyID + '/disabled', false);
			storeSubParameter('sonos-presets', presetPropertyID + '/conditions/enabled', true);

			storeSubParameter('sonos-presets', presetPropertyID + '/predefined', true);

			storeSubParameter('sonos-presets', presetPropertyID + '/roomID', roomID);
			storeSubParameter('sonos-presets', presetPropertyID + '/presetID', -1);
			// storeSubParameter('sonos-presets',
			// presetPropertyID+'/presetName', "");
			storeSubParameter('sonos-presets', presetPropertyID + '/actionType', SONOS_ACTION_COMMAND);
			storeSubParameter('sonos-presets', presetPropertyID + '/command', index);

			storeSubParameter('sonos-presets', presetPropertyID + '/volumeSet', false);
			storeSubParameter('sonos-presets', presetPropertyID + '/volume', 0);

			storeSubParameter('sonos-presets', presetPropertyID + '/audioName', "");
			storeSubParameter('sonos-presets', presetPropertyID + '/restartAfter', false);
			storeSubParameter('sonos-presets', presetPropertyID + '/setRepeat', false);
			storeSubParameter('sonos-presets', presetPropertyID + '/allRooms', false);
			storeSubParameter('sonos-presets', presetPropertyID + '/repeatTime', 0);
			storeSubParameter('sonos-presets', presetPropertyID + '/pickedRooms', 0);

			var arrayTriggers = predefinedRoomEvents[index];
			var trgSubIndex = 0;
			for (trgIndx in arrayTriggers) {
				lInfo.logln("trigger" + arrayTriggers[trgIndx].id + " -> " + presetPropertyID);
				/*
				 * storeSubParameter('sonos-presets',
				 * presetPropertyID+'/triggers/'+trgIndx+'/type',
				 * 'bus-zone-scene'); storeSubParameter('sonos-presets',
				 * presetPropertyID+'/triggers/'+trgIndx+'/zone', roomID);
				 * storeSubParameter('sonos-presets',
				 * presetPropertyID+'/triggers/'+trgIndx+'/group',
				 * arrayTriggers[trgIndx].group);
				 * storeSubParameter('sonos-presets',
				 * presetPropertyID+'/triggers/'+trgIndx+'/scene',
				 * arrayTriggers[trgIndx].id);
				 * storeSubParameter('sonos-presets',
				 * presetPropertyID+'/triggers/'+trgIndx+'/dsid', -1);
				 */

				var nSubTriggers = 4;
				if (arrayTriggers[trgIndx].group == 0) // for Scenes allready
					// assigned groupID as 0
					// we skip other sub
					// triggers
					// registrstration
					nSubTriggers = 1;

				for (iX = 1; iX <= nSubTriggers; iX++) {
					storeSubParameter('sonos-presets', presetPropertyID + '/triggers/' + trgSubIndex + '/type', 'bus-zone-scene');
					storeSubParameter('sonos-presets', presetPropertyID + '/triggers/' + trgSubIndex + '/scene', arrayTriggers[trgIndx].id);
					storeSubParameter('sonos-presets', presetPropertyID + '/triggers/' + trgSubIndex + '/dsuid', -1);

					var nTrgGroupID = arrayTriggers[trgIndx].group;
					var nTrgZone = roomID;
					switch (iX) {
					case 1:
						nTrgGroupID = arrayTriggers[trgIndx].group;
						nTrgZone = roomID;
						break;

					case 2:
						nTrgGroupID = arrayTriggers[trgIndx].group;
						nTrgZone = 0;
						break;

					case 3:
						nTrgGroupID = 0;
						nTrgZone = roomID;
						break;

					case 4:
						nTrgGroupID = 0;
						nTrgZone = 0;
						break;
					}
					storeSubParameter('sonos-presets', presetPropertyID + '/triggers/' + trgSubIndex + '/group', nTrgGroupID);
					storeSubParameter('sonos-presets', presetPropertyID + '/triggers/' + trgSubIndex + '/zone', nTrgZone);
					trgSubIndex++;
				}
			}
			var sPath = Property.getNode('sonos-presets/' + presetPropertyID).getPath();
			unregisterTrigger(sPath);
			registerTrigger(sPath, arrayTriggers[trgIndx].trigger, {});
			lInfo.logln("RegTrigger: " + sPath);
		}
	}
}

/**
 * @brief Function assigns player to dss Room
 * 
 * @param id
 *            [in] - player ID to assign room
 * @param roomID
 *            [in] - ID of room
 * 
 * @return
 */
function assignPlayerRoom(id, nRoomID) {
	try {
		var bCreatedDefaultPresets = false;
		var roomID = parseInt("" + unescape(nRoomID), 10);

		var oldZoneID = Property.getProperty('sonos-players/' + id + '/roomID');

		if (roomID > 0) {
			storeSubParameter('sonos-players', id + '/roomID', roomID);
			if (!testIfRoomHasDefaultTriggers(roomID)) {
				createDefaultRoomPresets(roomID);
				bCreatedDefaultPresets = true;
			}
		} else {
			storeSubParameter('sonos-players', id + '/roomID', 0);
		}

		// storeSubParameter('sonos-players', id+'/roomID', roomID);

		// storeSubParameterFull('/apartment/'+roomID+'/groups/group4/connectedDevices',
		// 1);
		updateZoneConnectedDevices();

		fShouldStore = true;
		return bCreatedDefaultPresets;
	} catch (e) {
		lInfo.logln("Error by saving assignPlayerRoom: " + e);
		for ( var i in e) {
			lInfo.logln(i + ': ' + e[i]);
		}
	}
	return false;
}

function disableAllPlayers() {
	var arrayAssignedRooms = new Array();
	var baseNode = Property.getNode('sonos-players');
	if (baseNode != null) {
		var oArrayNode = baseNode.getChildren();
		for ( var iIndex = 0; iIndex < oArrayNode.length; iIndex++) {
			lInfo.logln("player:" + iIndex);
			var oChildNode = oArrayNode[iIndex];
			var nRoomID = parseInt("" + oChildNode.getChild('roomID').getValue());

			if (!JSON.parse(oChildNode.getChild('disabled').getValue()) && nRoomID > 0) {
				arrayAssignedRooms.push({
					'deviceID' : unescape(oChildNode.getChild('deviceID').getValue()),
					'roomID' : nRoomID
				});
			}

			storeSubParameter('sonos-players', oChildNode.getChild('id').getValue() + '/notDiscovered', 'true');
			storeSubParameter('sonos-players', oChildNode.getChild('id').getValue() + '/disabled', 'true');
		}
	}

	return arrayAssignedRooms;
}

/**
 * @brief Function retrives full list of sonos devices from Sonos call
 *        http://ip:port//status/topology and stores them as players in PT
 * 
 * @param sSonosDeviceIP
 *            [in] - IP of sonos device
 * 
 * @return true if players are found and stored
 */
function getSonosDevices(sSonosDeviceIP, arrayAssignedRooms) {
	var curl_options = new Object({
		'debug' : 0,
		'connectTimeout' : URL_CONNECT_TIMEOUT
	});
	try {
		var h = new HTTP(curl_options, '');
		var result = h.get('http://' + sSonosDeviceIP + ':' + SONOS_PORT + '/status/topology', '');
		//
		if (result != null && result.body != null) {

			var aList = getXMLFields(result.body, 'ZonePlayer');

			var mainDevices = new Array();
			var subDevices = new Array();

			lInfo.logln("length=" + aList.length);

			for ( var iIndex = 0; iIndex < aList.length; iIndex++) {
				var sPlayerID = "";
				var bCoordinator = false;
				var sLocation = "";
				var sDeviceIP = "";
				var sPlayerSerial = "";
				var sPlayerName = "";
				var sRoomName = "";
				var sGroupID = "";

				var sData = aList[iIndex].data;
				lInfo.logln("pos:" + iIndex);

				if (sData != null) {
					matches = sData.match(/location=\'(.*?)\'/);
					if (matches != null) {
						sLocation = matches[1];
						lInfo.logln("matches:" + sLocation);

						// check if device is speaker
						var tmpData = h.get(sLocation);
						if (tmpData != null && tmpData.body != null && tmpData.body.indexOf("urn:schemas-upnp-org:service:MusicServices") > 0) {
							var matches = sLocation.match(/:\/\/(.[^:]+)/); // get
							// ip
							// from
							// full
							// url
							if (matches != null)
								sDeviceIP = matches[1];

							var matches = sData.match(/uuid=\'(.*?)\'/);
							if (matches != null)
								sPlayerID = matches[1];

							matches = tmpData.body.match(/<modelName>(.*?)<\/modelName>/);
							if (matches != null)
								sPlayerName = matches[1];

							matches = tmpData.body.match(/<serialNum>(.*?)<\/serialNum>/);
							if (matches != null)
								sPlayerSerial = matches[1];

							matches = sData.match(/group=\'(.*?)\'/);
							if (matches != null) {
								sGroupID = matches[1];
							}

							matches = sData.match(/coordinator=\'(.*?)\'/);
							if (matches != null) {
								bCoordinator = (matches[1].toLowerCase() == "true");
							}

							matches = sData.match(/<ZonePlayer[^>]*>(.*?)<\/ZonePlayer>/);
							if (matches != null)
								sRoomName = matches[1];

							if (bCoordinator) {
								mainDevices.push({
									playerID : sPlayerID,
									playerName : sPlayerName,
									serialNum : sPlayerSerial,
									playersInGroup : "",
									playersInGroupIP : "",
									sonosRoom : sRoomName,
									sonosGroup : sGroupID,
									roomID : 0,
									deviceIP : sDeviceIP
								});
							} else {
								subDevices.push({
									playerID : sPlayerID,
									playerName : sPlayerName,
									serialNum : sPlayerSerial,
									playersInGroup : "",
									sonosRoom : sRoomName,
									sonosGroup : sGroupID,
									deviceIP : sDeviceIP
								});
							}

							lInfo.logln("Found device: " + sPlayerID + " * " + sDeviceIP + " * " + sPlayerSerial + " * " + sRoomName + " * " + sGroupID);
						}
					}
				}
			}

			var bDevicesFound = false;

			// Storing new devices to Property tree
			if (mainDevices.length > 0) {
				bDevicesFound = true;

				if (arrayAssignedRooms == null)
					arrayAssignedRooms = disableAllPlayers();

				for ( var iIndex = 0; iIndex < mainDevices.length; iIndex++) {
					// search if has any sub devices
					var sSubDevices = "";
					var sSubDevicesSer = "";
					var sSubDevicesGroup = "";
					var sSubDevicesGroupIP = "";
					var sSubRooms = "";
					for ( var yIndex = 0; yIndex < subDevices.length; yIndex++) {
						if (subDevices[yIndex].sonosGroup == mainDevices[iIndex].sonosGroup) {
							if (sSubDevices.length > 0)
								sSubDevices = sSubDevices + ", ";
							sSubDevices = sSubDevices + subDevices[yIndex].playerName;

							if (sSubDevicesSer.length > 0)
								sSubDevicesSer = sSubDevicesSer + ", ";
							sSubDevicesSer = sSubDevicesSer + subDevices[yIndex].serialNum;

							if (sSubDevicesGroup.length > 0)
								sSubDevicesGroup = sSubDevicesGroup + ", ";
							sSubDevicesGroup = sSubDevicesGroup + subDevices[yIndex].playerID;

							if (sSubDevicesGroupIP.length > 0)
								sSubDevicesGroupIP = sSubDevicesGroupIP + ", ";
							sSubDevicesGroupIP = sSubDevicesGroupIP + subDevices[yIndex].deviceIP;

							if (subDevices[yIndex].sonosRoom != mainDevices[iIndex].sonosRoom) {
								if (sSubRooms.length > 0)
									sSubRooms = sSubRooms + ", ";
								sSubRooms = sSubRooms + subDevices[yIndex].sonosRoom;
							}
						}
					}

					if (sSubDevices.length > 0) {
						mainDevices[iIndex].playerName = mainDevices[iIndex].playerName + ", " + sSubDevices;
					}

					if (sSubDevicesSer.length > 0) {
						mainDevices[iIndex].serialNum = mainDevices[iIndex].serialNum + ", " + sSubDevicesSer;
					}

					mainDevices[iIndex].playersInGroup = "";
					if (sSubDevicesGroup.length > 0) {
						mainDevices[iIndex].playersInGroup = sSubDevicesGroup;
					}

					mainDevices[iIndex].playersInGroupIP = "";
					if (sSubDevicesGroupIP.length > 0) {
						mainDevices[iIndex].playersInGroupIP = sSubDevicesGroupIP;
					}

					if (sSubRooms.length > 0) {
						mainDevices[iIndex].sonosRoom = "Grouped [" + mainDevices[iIndex].sonosRoom + ", " + sSubRooms + "]";
					}

					lInfo.logln("Search group " + mainDevices[iIndex].sonosGroup)
					// mainDevices[iIndex].roomID = getStoredRoom(roomsList,
					// mainDevices[iIndex].sonosGroup);

					var playerID = storePlayer(mainDevices[iIndex], arrayAssignedRooms);
					if (playerID != INVALID_PLAYER_ID)
						resetStates(playerID, 0);
				}
			}

			return (mainDevices.length > 0);
		}
	} catch (e) {
		lInfo.logln("Error executing getSonosDevices " + e);
	}
	return false;
}

/**
 * @brief Function creates all triggers for Preset and Activities action
 * 
 * @param -
 * 
 * @return
 */
function updateAllTriggers() {
	var baseNode = Property.getNode('sonos-presets');
	if (baseNode != null) {
		var oArrayNode = baseNode.getChildren();
		if (oArrayNode != null) {
			for ( var iIndex = 0; iIndex < oArrayNode.length; iIndex++) {
				var oChildNode = oArrayNode[iIndex];
				var sPath = oChildNode.getPath();
				lInfo.logln("sonos-presets Create Trigger[" + iIndex + "] " + sPath);

				var sTriggerType = 'sonos_execute';
				var fPredefined = false;
				if (oChildNode.getChild('predefined') != null)
					fPredefined = JSON.parse(oChildNode.getChild('predefined').getValue());
				if ((fPredefined) && (oChildNode.getChild('actionType') != null && oChildNode.getChild('actionType').getValue() == SONOS_ACTION_COMMAND)
						&& (oChildNode.getChild('command') != null && oChildNode.getChild('command').getValue() == COMMAND_PAUSE)) {
					sTriggerType = 'sonos_execute_pause';
				}
				if ((fPredefined) && (oChildNode.getChild('actionType') != null && oChildNode.getChild('actionType').getValue() == SONOS_ACTION_COMMAND)
						&& (oChildNode.getChild('command') != null && oChildNode.getChild('command').getValue() == COMMAND_VOLUME_UP)) {
					sTriggerType = 'sonos_execute_volume';
				}
				if ((fPredefined) && (oChildNode.getChild('actionType') != null && oChildNode.getChild('actionType').getValue() == SONOS_ACTION_COMMAND)
						&& (oChildNode.getChild('command') != null && oChildNode.getChild('command').getValue() == COMMAND_VOLUME_DOWN)) {
					sTriggerType = 'sonos_execute_volume';
				}
				registerTrigger(oChildNode.getPath(), sTriggerType, {});
			}
		}
	}

	var baseNode = Property.getNode('sonos-activities');
	if (baseNode != null) {
		var oArrayNode = baseNode.getChildren();
		if (oArrayNode != null) {
			for ( var iIndex = 0; iIndex < oArrayNode.length; iIndex++) {
				var oChildNode = oArrayNode[iIndex];
				var sPath = oChildNode.getPath();
				lInfo.logln("sonos-activities Create Trigger[" + iIndex + "] " + sPath);
				registerActivityTrigger(oChildNode, 'sonos_execute');
			}
		}
	}
}

/**
 * @brief Function creates predefined triggers(predefinedStopActivities defined
 *        in file defined-values.js) for all activities
 * 
 * @param roomID
 *            [in] - ID of room to store triggers
 * 
 * @return
 */
var PREDEFINEDLASTVERSION=5;

function preparePredefinedActivities() {
	lInfo.logln('preparePredefinedActivities');
	var bPropertySaveExecuted = false;

	var baseNode = Property.getNode('sonos-predifined/activities');
	var bFillPredefined = true;
	var bDeleteOldPredefined = false;
	if (baseNode != null) { // activities allready added
		if (baseNode.getValue() >= PREDEFINEDLASTVERSION) // predefined latest version
			bFillPredefined = false;
	}

	if (bFillPredefined) {

		if ((Property.getNode('/scripts/remote-connectivity/OSPToken') != null) || (Property.getNode('/scripts/system-addon-remote-connectivity/OSPToken') != null)) {
			var sLangCode = Property.getProperty('/system/language/locale');
			

			var baseNode = Property.getNode('sonos-activities');
			if (baseNode != null) {
				var oArrayNode = baseNode.getChildren();
				for ( var iIndex = 0; iIndex < oArrayNode.length; iIndex++) {
					var oChildNode = oArrayNode[iIndex];
					lInfo.logln('checking ' + oChildNode.getPath());
					if (oChildNode.getChild('triggers') != null)
						if (oChildNode.getChild('predefined') != null)
							if (('' + oChildNode.getChild('predefined').getValue()) == 'true') {
								lInfo.logln('removing ' + oChildNode.getPath());
								unRegisterActivityTrigger(oChildNode);
								baseNode.removeChild(oChildNode.getName());
							}
				}
			}

			storeSubParameter('sonos-predifined', 'activities', PREDEFINEDLASTVERSION); // stored
			// predefined
			// and needs
			// to be
			// localized

			var activityPropertyID = getNewIdValue('sonos-activities');
			/* absent-event is everytime there, even without a cloud */
			storeSubParameter('sonos-activities', activityPropertyID + '/id', activityPropertyID);
			storeSubParameter('sonos-activities', activityPropertyID + '/volumeSet', '0');
			storeSubParameter('sonos-activities', activityPropertyID + '/volume', '0');
			storeSubParameter('sonos-activities', activityPropertyID + '/setRepeat', '0');
			storeSubParameter('sonos-activities', activityPropertyID + '/repeatTime', '0');
			storeSubParameter('sonos-activities', activityPropertyID + '/restartAfter', '0');
			storeSubParameter('sonos-activities', activityPropertyID + '/createSonosGroup', '0');
			storeSubParameter('sonos-activities', activityPropertyID + '/actionType', SONOS_ACTION_COMMAND);
			storeSubParameter('sonos-activities', activityPropertyID + '/disabled', 'false');
			storeSubParameter('sonos-activities', activityPropertyID + '/conditions/enabled', true);

			storeSubParameter('sonos-activities', activityPropertyID + '/predefined', 'true');
			storeSubParameter('sonos-activities', activityPropertyID + '/allRooms', 1);
			storeSubParameter('sonos-activities', activityPropertyID + '/command', COMMAND_STOP);
			storeSubParameter('sonos-activities', activityPropertyID + '/triggers/1/type', 'zone-scene'); // no
			// need
			// for
			// a
			// bus-call
			// scene
			// here
			storeSubParameter('sonos-activities', activityPropertyID + '/triggers/1/zone', 0);
			storeSubParameter('sonos-activities', activityPropertyID + '/triggers/1/group', 0);
			storeSubParameter('sonos-activities', activityPropertyID + '/triggers/1/scene', 72);
			storeSubParameter('sonos-activities', activityPropertyID + '/triggers/1/dsuid', -1);

			var node = Property.getNode('sonos-activities/' + activityPropertyID);
			registerActivityTrigger(node, 'sonos_execute');

			var arrayActivitiesToStore = new Array();
			var oCallback = function(defaultMediaEvents) {
				for ( var yIndex = 0; yIndex < defaultMediaEvents.length; yIndex++) {
					try {
						var command = defaultMediaEvents[yIndex];
						lInfo.logln('make ' + JSON.stringify(command));
						var activityPropertyID = getNewIdValue('sonos-activities');

						var otherParameters = function(name) { // just make a
							// closure because I
							// can
							var defaultAction = null;
							for ( var iIndex = 0; iIndex < predefinedActivitiesDefaultValues.length; iIndex++) {
								if (predefinedActivitiesDefaultValues[iIndex].type == name)
									return predefinedActivitiesDefaultValues[iIndex];
								if (predefinedActivitiesDefaultValues[iIndex].type == 'default')
									defaultAction = predefinedActivitiesDefaultValues[iIndex];
							}
							return defaultAction;
						}(command.EventType);
						lInfo.logln('huhu ' + JSON.stringify(otherParameters));
						storeSubParameter('sonos-activities', activityPropertyID + '/id', activityPropertyID);
						storeSubParameter('sonos-activities', activityPropertyID + '/volumeSet', 'true');
						storeSubParameter('sonos-activities', activityPropertyID + '/volume', otherParameters.volume);
						storeSubParameter('sonos-activities', activityPropertyID + '/setRepeat', ((otherParameters.repeat > 0) ? 'true' : '0'));
						storeSubParameter('sonos-activities', activityPropertyID + '/repeatTime', '' + otherParameters.repeat);
						storeSubParameter('sonos-activities', activityPropertyID + '/restartAfter', (otherParameters.restart ? '1' : '0'));
						storeSubParameter('sonos-activities', activityPropertyID + '/createSonosGroup', 1);
						storeSubParameter('sonos-activities', activityPropertyID + '/actionType', SONOS_ACTION_LIBRARY);
						storeSubParameter('sonos-activities', activityPropertyID + '/disabled', 'false');
						storeSubParameter('sonos-activities', activityPropertyID + '/conditions/enabled', true);

						storeSubParameter('sonos-activities', activityPropertyID + '/predefined', 'true');
						storeSubParameter('sonos-activities', activityPropertyID + '/allRooms', 1);
						storeSubParameter('sonos-selected-preset', 'tmp-queue', "");

						storeSubParameter('sonos-activities', activityPropertyID + '/mediaFileTitle', ((command.MediaFileName==null)?'':command.MediaFileName));
						storeSubParameter('sonos-activities', activityPropertyID + '/mediaGroupGUID', ''); // maybe
						// we
						// must
						// fetch
						// that
						// afterwarts
						storeSubParameter('sonos-activities', activityPropertyID + '/mediaGroupSubGUID', '');
						storeSubParameter('sonos-activities', activityPropertyID + '/mediaFileGUID', command.MediaFileGUID);
						storeSubParameter('sonos-activities', activityPropertyID + '/nowSingleUrl', command.URLPath);
						storeSubParameter('sonos-activities', activityPropertyID + '/nowSingleLength', '');

						if (command.EventType.indexOf('apartment_scene_')>-1) {
							storeSubParameter('sonos-activities', activityPropertyID + '/triggers/1/type', 'zone-scene'); // no
							// need
							// for
							// a
							// bus-call
							// scene
							// here
							storeSubParameter('sonos-activities', activityPropertyID + '/triggers/1/zone', 0);
							storeSubParameter('sonos-activities', activityPropertyID + '/triggers/1/group', 0);
							storeSubParameter('sonos-activities', activityPropertyID + '/triggers/1/scene', parseInt(command.EventType.split('_')[2], 10));
							storeSubParameter('sonos-activities', activityPropertyID + '/triggers/1/dsuid', -1);
							if (otherParameters.hasUndo) {
								storeSubParameter('sonos-activities', activityPropertyID + '/undoCallScene/triggers/1/type', 'undo-zone-scene'); // no
								// need
								// for
								// a
								// bus-call
								// scene
								// here
								storeSubParameter('sonos-activities', activityPropertyID + '/undoCallScene/triggers/1/zone', 0);
								storeSubParameter('sonos-activities', activityPropertyID + '/undoCallScene/triggers/1/group', 0);
								storeSubParameter('sonos-activities', activityPropertyID + '/undoCallScene/triggers/1/scene', parseInt(command.EventType.split('_')[2], 10));
								storeSubParameter('sonos-activities', activityPropertyID + '/undoCallScene/triggers/1/dsuid', -1);
							}
						}
						if (command.EventType.indexOf('apartment_state_')>-1) {
							storeSubParameter('sonos-activities', activityPropertyID + '/triggers/1/type', 'state-change');
							storeSubParameter('sonos-activities', activityPropertyID + '/triggers/1/name', command.EventType.split('_')[2]);
							storeSubParameter('sonos-activities', activityPropertyID + '/triggers/1/state', command.EventType.split('_')[3]);
							if (otherParameters.hasUndo) {
								storeSubParameter('sonos-activities', activityPropertyID + '/undoCallScene/triggers/1/type', 'state-change');
								storeSubParameter('sonos-activities', activityPropertyID + '/undoCallScene/triggers/1/name', command.EventType.split('_')[2]);
								storeSubParameter('sonos-activities', activityPropertyID + '/undoCallScene/triggers/1/state', (command.EventType.split('_')[3] == 'active') ? 'inactive' : 'active');
							}
						}
						var node = Property.getNode('sonos-activities/' + activityPropertyID);
						registerActivityTrigger(node, 'sonos_execute');
					} catch (e) {
						lInfo.logln('Error ' + e);
					}
						
				}
				var syncLibrary=function(oData) {
					var fBaseUID=[];

					  
					lInfo.logln('Syncing medialib with predefined');
					if (oData!=null) {
				        if (oData.ReturnCode != null && oData.ReturnCode == 0) {
				          mediaObject = oData.Response.MediaLibraryTreeList;
				          for (var jIndex=0;jIndex<mediaObject.length;jIndex++) {
							  var mediaFile=mediaObject[jIndex];
							  if (mediaFile.MediaLibraryTreeBranchParentGUID==null)
								  fBaseUID.push(mediaFile.MediaLibraryTreeBranchGUID);								 
						  }
				          
				          var baseNode = Property.getNode('sonos-activities').getChildren();
						  if (baseNode != null)
							  for (var iIndex=0;iIndex<baseNode.length;iIndex++) {
								  var childNode=baseNode[iIndex];
								  if (childNode.getChild('actionType').getValue()==SONOS_ACTION_LIBRARY) 
									  if (childNode.getChild('mediaGroupGUID').getValue()==''){
										  var fileGUID=childNode.getChild('mediaFileGUID').getValue();
										  for (var jIndex=0;jIndex<mediaObject.length;jIndex++) {
											  for (var kIndex=0;kIndex<mediaObject[jIndex].MediaFileList.length;kIndex++) {
												  var mediaFile=mediaObject[jIndex].MediaFileList[kIndex];
												  if (mediaFile.MediaLibraryTreeMediaFileGUID==fileGUID) {
													  
													  if (fBaseUID.indexOf(mediaObject[jIndex].MediaLibraryTreeBranchParentGUID)>-1) {
													      storeSubParameter(childNode.getPath(),'mediaFileTitle',mediaFile.MediaFileName);
														  storeSubParameter(childNode.getPath(),'mediaGroupGUID',mediaObject[jIndex].MediaLibraryTreeBranchGUID); // beim t�rklingel muss die subid = groupid sein, subid muss leer sein
														  storeSubParameter(childNode.getPath(),'mediaGroupSubGUID',fileGUID);
													  } else {
													  	  storeSubParameter(childNode.getPath(),'mediaFileTitle',mediaObject[jIndex].BranchName + ' - ' + mediaFile.MediaFileName);
														  storeSubParameter(childNode.getPath(),'mediaGroupGUID',mediaObject[jIndex].MediaLibraryTreeBranchParentGUID); // beim t�rklingel muss die subid = groupid sein, subid muss leer sein
														  storeSubParameter(childNode.getPath(),'mediaGroupSubGUID',mediaObject[jIndex].MediaLibraryTreeBranchGUID);
													  }
												  }
											  }
										  }
									  }
							  }
				        }
				      }
					  lInfo.logln('Syncing medialib done');
					  Property.store();
				}; 
				
				cloudService.fetchMediaLibrary(syncLibrary);
				
			};
			fetchMediaDefaultConfiguration(oCallback);

		}
	}

	return;
}

function updateZoneConnectedDevices() {
	// connectedDevices
	var zones = getZones();
	for ( var iIndex = 0; iIndex < zones.length; iIndex++) {
		if (zones[iIndex].id === 0) {
			continue;
		}
		try {
			zones[iIndex].removeConnectedDevice(4);
		} catch (e) {
			storeSubParameterFull('/apartment/zones/zone' + zones[iIndex].id + '/groups/group4/connectedDevices', 0);
		}
	}

	baseNode = Property.getNode('sonos-players');
	if (baseNode != null) {
		var oArrayNode = baseNode.getChildren();
		for ( var iIndex = 0; iIndex < oArrayNode.length; iIndex++) {
			var oChildNode = oArrayNode[iIndex];
			if (!JSON.parse(oChildNode.getChild('notDiscovered').getValue())) {
				if (oChildNode.getChild('roomID') != null) {
					var nRoomID = parseInt("" + (oChildNode.getChild('roomID').getValue()), 10);
					if (nRoomID > 0) {
						lInfo.logln("Activate connectedDevices for zoneID=" + nRoomID);
						try {
							getZoneByID(nRoomID).addConnectedDevice(4);
						} catch (e) {
							lInfo.logln('Failed to execute addConnectedDevice. Trying to assign value manually');
							storeSubParameterFull('/apartment/zones/zone' + nRoomID + '/groups/group4/connectedDevices', 1);
						}
					}
				}
			}
		}
	}
}

function startCloudService(bRetryConnection) {
	try {
		lInfo.logln('start startService');
		storeSubParameter('media-service', 'isServiceConnected', 0);
		storeSubParameter('media-service', 'isConnectivityAvailable', cloudService.isRemoteConnectivityReady());

		if ((Property.getNode('/scripts/remote-connectivity/OSPToken') == null) && (Property.getNode('/scripts/system-addon-remote-connectivity/OSPToken') == null)) {
			if (bRetryConnection) {
				lInfo.logln('Remote Connectivity not ready! New try in 5 minutes.');
				var evt = new Event("sonos-controller.save", {
					actions : 'startService',
					'time' : '+300'
				});
				evt.raise();
			}
			return;
		} else {
			cloudService.startService(function(oData) {
				if (oData.ReturnMessage != "OK") {
					if (bRetryConnection) {
						lInfo.logln('StartMediaService failed! New try in 5 minutes.');
						var evt = new Event("sonos-controller.save", {
							actions : 'startService',
							'time' : '+300'
						});
						evt.raise();
					}
				} else {
					storeSubParameter('media-service', 'isServiceConnected', 1);
					lInfo.logln('StartMediaService is connected.');
					preparePredefinedActivities();
				}
			});
		}
	} catch (e) {
		lWarn.logln('startService error ' + e);
	}
}

function deleteUnusedSonosQueues(sSonosIP) {
	try {
		// deleting all temporary created playlists
		var curl_options = new Object({
			'debug' : 0,
			'connectTimeout' : 2000
		});

		var arrayLists = upnpGetSonosPlaylist([ sSonosIP ], 1000, true, curl_options);
		if (arrayLists != null && arrayLists.length > 0) {
			for ( var iIndex = 0; iIndex < arrayLists.length; iIndex++) {
				upnpDeleteQueueFromList(sSonosIP, arrayLists[iIndex].listID, curl_options);
			}
		}
	} catch (e) {
		lWarn.logln('deleteUnusedQueues error ' + e);
	}
}

function autoDiscoveryStep() {
	if (raisedEvent.parameter.baseIp == null) {
		lWarn.logln('missing parameter baseIp');
		return;
	}
	if (raisedEvent.parameter.stepSize == null) {
		lWarn.logln('missing parameter stepSize');
		return;
	}
	if (raisedEvent.parameter.step == null) {
		lWarn.logln('missing parameter step');
		return;
	}
	lInfo.logln('making autodiscovery for ' + raisedEvent.parameter.baseIp + ' with ' + raisedEvent.parameter.stepSize + ' on step ' + raisedEvent.parameter.step);
	var sIP = raisedEvent.parameter.baseIp;
	var startIndex = raisedEvent.parameter.step;

	var arrayActiveRooms = new Array();
	if (raisedEvent.parameter.activeRooms != null && raisedEvent.parameter.activeRooms != "")
		arrayActiveRooms = JSON.parse(raisedEvent.parameter.activeRooms);

	var stopIndex = raisedEvent.parameter.step + raisedEvent.parameter.stepSize;
	if (stopIndex > 256)
		stopIndex = 256;
	var sResult = "";

	var curl_options = new Object({
		'debug' : 0,
		'connectTimeout' : URL_CONNECT_TIMEOUT
	});
	var h = new HTTP(curl_options, '');

	for ( var iIndex = startIndex; iIndex < stopIndex; iIndex++) {
		var sSearchedIP = sIP + iIndex;

		var sUrl = "http://" + sSearchedIP + ":" + SONOS_PORT + "/xml/device_description.xml";
		lInfo.logln("Testurl " + sUrl);

		var result = h.get(sUrl);
		if (result != null && result.body != null) {
			if (result.body.indexOf("schemas-upnp-org") != -1 && result.body.indexOf("sonos.com</manufacturerURL>") != -1) {
				lInfo.logln("Result: On IP is Sonos device - Starting full discovery");
				if (getSonosDevices(sSearchedIP, arrayActiveRooms)) {
					lInfo.logln("Result: Devices discovered. Finishing autodiscovery.");
					sResult = sIP;
					updateZoneConnectedDevices();
					deleteUnusedSonosQueues(sIP);

					clearSonosGrouping();
					preparePredefinedActivities();
					var evt = new Event("sonos-autodiscover-finished", {
						ip : sResult
					});
					evt.raise();
					return;
				}
			}
		}
		result = null;
		lInfo.logln("----------");
	}
	if (stopIndex < 256) {
		var sActiveRooms = "";
		if (arrayActiveRooms != null && arrayActiveRooms.length > 0)
			sActiveRooms = JSON.stringify(arrayActiveRooms);

		var evt = new Event("sonos-controller.stepdiscovery", {
			baseIp : sIP,
			stepSize : 5,
			step : stopIndex,
			activeRooms : sActiveRooms
		});
		evt.raise();
	} else {
		var evt = new Event("sonos-autodiscover-finished", {
			ip : "-"
		});
		evt.raise();
	}
}

lInfo.logln("Loading player.js -> " + raisedEvent.name);
if (raisedEvent.name === 'running') {
	lInfo.logln("running sonos players.js");
	Property.load();
    convertPropertyTreeEntries();
	var oPlayers = getAllPlayers(true);
	if (oPlayers.length != 0) {
		var oValue = [];
		for ( var iIndex = 0; iIndex < oPlayers.length; iIndex++) {
			oValue.push({
				"inList" : 1,
				"ipAddress" : oPlayers[iIndex]
			});
		}
		new Event('sonos-controller.startdiscovery', {
			"actions" : "AutoDiscover",
			"value" : JSON.stringify(oValue)
		}).raise();
	}

	lInfo.logln("running sonos players.js end");
} else if (raisedEvent.name == 'model_ready') {
	lInfo.logln("model_ready sonos players.js");

	// start cloud service
	startCloudService(true);

	updateAllTriggers();

	updateZoneConnectedDevices();

	var sActiveSonosIP = "";
	var arrayUnfinishedActivities = new Array();
	var arrayUnfinishedActivitiesIDs = new Array();
	/*
	 * var baseNode = Property.getNode('sonos-players'); if (baseNode != null) {
	 * var oArrayNode = baseNode.getChildren(); for ( var iIndex = 0; iIndex <
	 * oArrayNode.length; iIndex++) { var oChildNode = oArrayNode[iIndex]; //
	 * resetStates(oChildNode.getChild('id').getValue(), 0); if
	 * (!JSON.parse(oChildNode.getChild('disabled').getValue())) { if
	 * (sActiveSonosIP == "") sActiveSonosIP =
	 * oChildNode.getChild('ipAddress').getValue();
	 * 
	 * var tmpActivity =
	 * getCurrentActiveActivity(oChildNode.getChild('id').getValue());
	 * lInfo.logln("tmpActivity:" + JSON.stringify(tmpActivity)); if
	 * (tmpActivity.nActiveActivityId != INVALID_ACTIVITY_ID) { if
	 * (arrayUnfinishedActivitiesIDs.indexOf(tmpActivity.nActiveActivityId) < 0) {
	 * arrayUnfinishedActivitiesIDs.push(tmpActivity.nActiveActivityId);
	 * arrayUnfinishedActivities.push(tmpActivity); } } } } }
	 */	
		
	lInfo.logln(">>>>>>>arrayUnfinishedActivities:" + JSON.stringify(arrayUnfinishedActivities));

	if (arrayUnfinishedActivities.length > 0) {
		var curl_options = new Object({
			'debug' : 0,
			'connectTimeout' : 200
		});
		for ( var iIndex = 0; iIndex < arrayUnfinishedActivities.length; iIndex++) {
			var activity = arrayUnfinishedActivities[iIndex];
			lInfo.logln("restart executing:" + '/scripts/sonos-controller/sonos-activities/' + activity.nActiveActivityId);
			executeActivity('/scripts/sonos-controller/sonos-activities/' + activity.nActiveActivityId,// +
			activity.eventId, activity.eventType, false, curl_options);
		}
	} else {
		if (sActiveSonosIP != "")
			deleteUnusedSonosQueues(sActiveSonosIP);
		clearSonosGrouping();
	}

	lInfo.logln("model_ready sonos players.js end");
} else if (raisedEvent.name == 'sonos-controller.save') {
	if (raisedEvent.parameter.actions == 'preparePredefinedActivities') {
		preparePredefinedActivities();
	}
	if (raisedEvent.parameter.actions == 'assignRoom') {
		var sObject = JSON.parse(unescape(raisedEvent.parameter.value));
		lInfo.logln("Assign room:" + sObject.playersList.roomID);
		var bCreateDefaultPresets = false;
		if (sObject != null)
			bCreateDefaultPresets = assignPlayerRoom(sObject.id, sObject.playersList.roomID);

		nAssignedRoom = -1;
		if (bCreateDefaultPresets)
			nAssignedRoom = sObject.playersList.roomID;

		// interactive save should wait until storage is finished, so it can not
		// been delayed
		Property.store();
		fShouldStore = false;
		var evt = new Event("sonos-controller-saved", {
			nNewRoomID : nAssignedRoom
		});
		evt.raise();
	} else if (raisedEvent.parameter.actions == 'deletePreset') {
		var oIDObject = JSON.parse(unescape(raisedEvent.parameter.id));

		if (typeof (oIDObject) != 'object') {
			oIDObject = [ oIDObject ];
		}

		for ( var sKey in oIDObject) {
			var baseNode = Property.getNode("sonos-presets");
			if (baseNode != null) {
				var sPath = Property.getNode('sonos-presets/' + oIDObject[sKey]).getPath();
				lInfo.logln("Delete trigger " + sPath);
				unregisterTrigger(sPath);
				lInfo.logln("Delete trigger finished");
				baseNode.removeChild(oIDObject[sKey]);
			}
		}
		// interactive save should wait until storage is finished, so it can not
		// been delayed
		Property.store();
		fShouldStore = false;

		var evt = new Event("sonos-controller-saved", {});
		evt.raise();
	} else if (raisedEvent.parameter.actions == 'deleteActivity') {
		var oIDObject = JSON.parse(unescape(raisedEvent.parameter.id));

		if (typeof (oIDObject) != 'object') {
			oIDObject = [ oIDObject ];
		}

		for ( var sKey in oIDObject) {
			var baseNode = Property.getNode("sonos-activities");
			if (baseNode != null) {
				var oActivityNode = Property.getNode('sonos-activities/' + oIDObject[sKey]);
				if (oActivityNode != null) {
					var sPath = oActivityNode.getPath();
					lInfo.logln("Delete trigger " + sPath);
					unRegisterActivityTrigger(Property.getNode('sonos-activities/' + oIDObject[sKey]));
					lInfo.logln("Delete trigger finished");

					// if text2speech we remove created text voice
					if (oActivityNode.getChild('actionType') != null && oActivityNode.getChild('actionType').getValue() == SONOS_ACTION_TTS) {
						if (oActivityNode.getChild('t2Sguid') != null && oActivityNode.getChild('t2Sguid').getValue() != "") {
							cloudService.deleteTextSound(oActivityNode.getChild('t2Sguid').getValue(), function(oData) {
								lInfo.logln("Deleted activity voice file");
							});
						}
					}

					lInfo.logln("Delete child " + oIDObject[sKey]);
					baseNode.removeChild(oIDObject[sKey]);
				}
			}
		}

		// interactive save should wait until storage is finished, so it can not
		// been delayed
		Property.store();
		fShouldStore = false;

		var evt = new Event("sonos-controller-saved", {});
		evt.raise();
	} else if (raisedEvent.parameter.actions == 'storePreset') {
		var objData = JSON.parse(unescape(raisedEvent.parameter.value));
		if (objData != null) {
			lInfo.logln("Store preset");
			storePreset(objData);
		}

		// interactive save should wait until storage is finished, so it can not
		// been delayed
		Property.store();
		fShouldStore = false;

		var evt = new Event("sonos-controller-saved", {});
		evt.raise();
	} else if (raisedEvent.parameter.actions == 'storeActivity') {
		var objData = JSON.parse(unescape(raisedEvent.parameter.value));
		var sInfo = "";
		if (objData != null) {
			lInfo.logln("Store activity");
			sInfo = storeActivity(objData, true);
		}
		lInfo.logln("Store activity create saved event [" + sInfo + "]");

		Property.store();
		fShouldStore = false;

		var evt = new Event("sonos-controller-saved", {
			value : JSON.stringify(sInfo)
		});
		evt.raise();
	} else if (raisedEvent.parameter.actions == 'enablePreset') {
		var oIDObject = JSON.parse(unescape(raisedEvent.parameter.id));
		if (typeof (oIDObject) != 'object')
			oIDObject = [ oIDObject ];

		for ( var sKey in oIDObject) {
			nPresetID = parseInt("" + oIDObject[sKey], 10);
			if (nPresetID > -1) {
				storeSubParameter('sonos-presets', nPresetID + '/disabled', false);
				storeSubParameter('sonos-presets', nPresetID + '/conditions/enabled', true);
				lInfo.logln("Enable Preset " + sPath);
			}
		}

		// interactive save should wait until storage is finished, so it can not
		// been delayed
		Property.store();
		fShouldStore = false;

		var evt = new Event("sonos-controller-saved", {});
		evt.raise();
	} else if (raisedEvent.parameter.actions == 'disablePreset') {
		var oIDObject = JSON.parse(unescape(raisedEvent.parameter.id));
		if (typeof (oIDObject) != 'object')
			oIDObject = [ oIDObject ];

		for ( var sKey in oIDObject) {
			nPresetID = parseInt("" + oIDObject[sKey], 10);
			if (nPresetID > -1) {
				storeSubParameter('sonos-presets', nPresetID + '/disabled', true);
				storeSubParameter('sonos-presets', nPresetID + '/conditions/enabled', false);
			}
		}
		// interactive save should wait until storage is finished, so it can not
		// been delayed
		Property.store();
		fShouldStore = false;

		var evt = new Event("sonos-controller-saved", {});
		evt.raise();
	} else if (raisedEvent.parameter.actions == 'enableActivity') {
		var oIDObject = JSON.parse(unescape(raisedEvent.parameter.id));
		if (typeof (oIDObject) != 'object')
			oIDObject = [ oIDObject ];

		for ( var sKey in oIDObject) {
			nActivityID = parseInt("" + oIDObject[sKey], 10);
			if (nActivityID > -1) {
				storeSubParameter('sonos-activities', nActivityID + '/disabled', false);
				storeSubParameter('sonos-activities', nActivityID + '/conditions/enabled', true);
			}
		}
		// interactive save should wait until storage is finished, so it can not
		// been delayed
		Property.store();
		fShouldStore = false;

		var evt = new Event("sonos-controller-saved", {});
		evt.raise();
	} else if (raisedEvent.parameter.actions == 'disableActivity') {
		var oIDObject = JSON.parse(unescape(raisedEvent.parameter.id));
		if (typeof (oIDObject) != 'object')
			oIDObject = [ oIDObject ];

		for ( var sKey in oIDObject) {
			nActivityID = parseInt("" + oIDObject[sKey], 10);
			if (nActivityID > -1) {
				storeSubParameter('sonos-activities', nActivityID + '/disabled', true);
				storeSubParameter('sonos-activities', nActivityID + '/conditions/enabled', false);
			}
		}
		// interactive save should wait until storage is finished, so it can not
		// been delayed
		Property.store();
		fShouldStore = false;

		var evt = new Event("sonos-controller-saved", {});
		evt.raise();
	} else if (raisedEvent.parameter.actions == 'enablePlayer') {
		var oIDObject = JSON.parse(unescape(raisedEvent.parameter.id));
		if (typeof (oIDObject) != 'object')
			oIDObject = [ oIDObject ];

		for ( var sKey in oIDObject) {
			nPlayerID = parseInt("" + oIDObject[sKey], 10);
			if (nPlayerID > -1) {
				storeSubParameter('sonos-players', nPlayerID + '/disabled', false);
			}
		}
		updateZoneConnectedDevices();
		// interactive save should wait until storage is finished, so it can not
		// been delayed
		Property.store();
		fShouldStore = false;

		var evt = new Event("sonos-controller-saved", {});
		evt.raise();
	} else if (raisedEvent.parameter.actions == 'disablePlayer') {
		var oIDObject = JSON.parse(unescape(raisedEvent.parameter.id));
		if (typeof (oIDObject) != 'object')
			oIDObject = [ oIDObject ];

		for ( var sKey in oIDObject) {
			var nPlayerID = parseInt("" + oIDObject[sKey], 10);
			if (nPlayerID > -1) {
				storeSubParameter('sonos-players', nPlayerID + '/disabled', true);
				lInfo.logln("Disabled Player " + sPath);
			}
		}
		updateZoneConnectedDevices();
		// interactive save should wait until storage is finished, so it can not
		// been delayed
		Property.store();
		fShouldStore = false;

		var evt = new Event("sonos-controller-saved", {});
		evt.raise();
	} else if (raisedEvent.parameter.actions == 'deletePlayer') {
		var oIDObject = JSON.parse(unescape(raisedEvent.parameter.id));
		if (typeof (oIDObject) != 'object')
			oIDObject = [ oIDObject ];

		for ( var sKey in oIDObject) {
			var baseNode = Property.getNode("sonos-players");
			if (baseNode != null) {
				// var sPath =
				// Property.getNode('sonos-players/'+oIDObject[sKey]).getPath();
				baseNode.removeChild(oIDObject[sKey]);
			}
		}
		updateZoneConnectedDevices();
		// interactive save should wait until storage is finished, so it can not
		// been delayed
		Property.store();
		fShouldStore = false;

		var evt = new Event("sonos-controller-saved", {});
		evt.raise();
	} else if (raisedEvent.parameter.actions == 'startService') {
		startCloudService(true);
	}
} else if (raisedEvent.name == 'sonos-controller.startdiscovery') {
	if (raisedEvent.parameter.actions == 'discover') {
		var sObject = JSON.parse(unescape(raisedEvent.parameter.value));
		lInfo.logln("startdiscovery=" + sObject.discoverIP);
		var sResult = "";
		if (getSonosDevices(sObject.discoverIP, null)) {
			sResult = sObject.discoverIP;

			updateZoneConnectedDevices();

			clearSonosGrouping();
			preparePredefinedActivities();
			fShouldStore=false; 
		}

		var evt = new Event("sonos-discover-finished", {
			ip : sResult
		});
		evt.raise();
		lInfo.logln("End discovery");
	} else if (raisedEvent.parameter.actions == 'AutoDiscover') {
		var arrayAdresses = JSON.parse(unescape(raisedEvent.parameter.value));
		var curl_options = new Object({
			'debug' : 0,
			'connectTimeout' : URL_CONNECT_TIMEOUT
		});
		lInfo.logln("AutoDiscover " + unescape(raisedEvent.parameter.value));
		var h = new HTTP(curl_options, '');

		arrayActiveRoomsBeforeDiscovery = disableAllPlayers();

		for ( var iIndex = 0; iIndex < arrayAdresses.length; iIndex++) {
			var oChild = arrayAdresses[iIndex];
			var sIP = oChild.ipAddress;

			if (oChild.inList == 1) { // testing on existing IP from grid
				lInfo.logln("Autodiscovery on existing IP " + sIP);

				var sUrl = "http://" + sIP + ":" + SONOS_PORT + "/xml/device_description.xml";
				lInfo.logln("Testurl " + sUrl);

				var result = null;
				result = h.get(sUrl);

				if (result != null && result.body != null) {
					lInfo.logln("Result: IP returns result");
					if (result.body.indexOf("schemas-upnp-org") != -1 && result.body.indexOf("sonos.com</manufacturerURL>") != -1) {
						lInfo.logln("Result: On IP is Sonos device - Starting full discovery");
						if (getSonosDevices(sIP, arrayActiveRoomsBeforeDiscovery)) {
							lInfo.logln("Result: Devices discovered. Finishing autodiscovery.");
							sResult = sIP;
							lInfo.logln("Create predefined activities");
							updateZoneConnectedDevices();
							deleteUnusedSonosQueues(sIP);

							clearSonosGrouping();
							preparePredefinedActivities();
							var evt = new Event("sonos-autodiscover-finished", {
								ip : sIP
							});
							evt.raise();

							break;
						}
					}
				}
			} else { // testing dSS IP and range 1-255
				var aIP = sIP.split('.');
				if (aIP.length == 4)
					sIP = aIP[0] + "." + aIP[1] + "." + aIP[2] + ".";

				var sActiveRooms = "";
				if (arrayActiveRoomsBeforeDiscovery != null)
					sActiveRooms = JSON.stringify(arrayActiveRoomsBeforeDiscovery);

				var evt = new Event("sonos-controller.stepdiscovery", {
					baseIp : sIP,
					stepSize : 5,
					step : 1,
					activeRooms : sActiveRooms
				});
				evt.raise();
			}
		}
	}
} else if (raisedEvent.name == 'sonos-controller.stepdiscovery') {
	autoDiscoveryStep();
}

else if (raisedEvent.name == 'sonos-controller.getdata') {
	if (raisedEvent.parameter.actions == 'getSonosQueue') {
		lInfo.logln("Inside getSonosQueue");

		var oObject = JSON.parse(unescape(raisedEvent.parameter.value));

		var curl_options = new Object({
			'debug' : 0,
			'connectTimeout' : 3000
		/* in mseconds */
		});

		var data = "";
		var arrayLists = upnpGetCurrentQueue(oObject, 100, curl_options);
		if (arrayLists != null) {
			data = JSON.stringify(arrayLists);
			lInfo.logln(data);
			// used only to communicate with ui, so no persistence are needed
			Property.setProperty('sonos-selected-preset/tmp-queue', escape(data));
		}

		var evt = new Event("sonos-discover-return-sonos-queue", {
			returnValue : data
		});
		evt.raise();
	} else if (raisedEvent.parameter.actions == 'getSonosTrack') {

		var oObject = JSON.parse(unescape(raisedEvent.parameter.value));
		lInfo.logln("Inside getSonosTrack for IP:" + oObject);

		var curl_options = new Object({
			'debug' : 0,
			'connectTimeout' : 2000
		/* in mseconds */
		});

		var data = "";
		var objCurrentTrack = upnpGetCurrentTrack(oObject, curl_options);
		if (objCurrentTrack != null) {
			data = JSON.stringify(objCurrentTrack);
			lInfo.logln(data);
		}
		var evt = new Event("sonos-discover-return-sonos-track", {
			returnValue : data
		});
		evt.raise();
	} else if (raisedEvent.parameter.actions == 'getSonosPlaylists') {
		lInfo.logln("Inside getSonosPlaylists" + raisedEvent.parameter.value);

		var oObject = JSON.parse(unescape(raisedEvent.parameter.value));

		var curl_options = new Object({
			'debug' : 0,
			'connectTimeout' : 2000
		});

		var arrayLists = upnpGetSonosPlaylist(getAllPlayers(), NR_ITEMS_FETCHED_FROM_PLAYLIST, false, curl_options);
		saveSonosPlayList(arrayLists);

		arrayLists = upnpGetSonosFavorites(getAllPlayers(), NR_ITEMS_FETCHED_FROM_FAVORITES, curl_options);
		saveSonosFavorites(arrayLists);

		if (oObject.loadMediaSpeech != null && JSON.parse(oObject.loadMediaSpeech)) {
			// start cloud service
			startCloudService(false);

			var returnData = new Object();
			cloudService.fetchMediaLibrary(function(oData) {
				var mediaObject = null;
				if (oData != null) {
					if (oData.ReturnCode != null && oData.ReturnCode == 0) {
						mediaObject = JSON.stringify(oData.Response.MediaLibraryTreeList);
					}
				} else
					lInfo.logln("fetchMediaLibrary empty return");
				returnData.mediaLibrary = mediaObject;

				cloudService.fetchText2SpeechConf(function(oData) {
					var mediaConfig = new Object();
					if (oData != null) {
						if (oData.ReturnCode != null && oData.ReturnCode == 0) {
							mediaConfig.config = oData.Response.LanguageCodeList;
							mediaConfig.maxCalls = oData.Response.MaxTextToSpeechCalls;
							mediaConfig.maxTextLength = oData.Response.MaxTextToSpeechLength;
						}
					} else
						lInfo.logln("fetchSpeechConfiguration empty return");

					returnData.text2Speech = mediaConfig;
					var evt = new Event("sonos-controller-playlist-stored", {
						value : JSON.stringify(returnData)
					});
					evt.raise();
				});
			});
		} else {
			var evt = new Event("sonos-controller-playlist-stored", {});
			evt.raise();
		}
	}
	if (raisedEvent.parameter.actions == 'getCurrentPresetRoom') {
		lInfo.logln("Inside getCurrentPresetRoom");

		var baseNode = Property.getNode('sonos-selected-preset');
		var roomID = -1;
		if (baseNode != null) {
			roomID = baseNode.getChild('roomID').getValue();
		}

		var data = escape(JSON.stringify(objCurrentTrack));
		var evt = new Event("sonos-return-preset-room", {
			returnValue : roomID
		});
		evt.raise();
	}

} else if (raisedEvent.name == 'sonos-controller.storePropertyTree') {
	lInfo.logln("Storing to Propery Tree");
	fShouldStore = true;
}

if (fShouldStore)
	Property.store();
